@@ -14,11 +14,6 @@ from absbox.local.util import mkTag,DC,mkTs,query,consolStmtByDate,aggStmtByDate
 from absbox.local.component import *
 
 
-def mkCf(x):
-    if len(x)==0:
-        return None
-    else:
-        return [ mkTag(("MortgageFlow",_x+([0.0]*5))) for _x in x]
 
 def readIssuance(pool):
     if '发行' not in pool.keys():
@@ -29,130 +24,12 @@ def readIssuance(pool):
     r = {} 
     for k,v in pool['发行'].items():
         r[issuanceField[k]] = v
-
     return r
 
-def mkCollection(xs):
-    sourceMapping = {"利息回款": "CollectedInterest"
-                    , "本金回款": "CollectedPrincipal"
-                    , "早偿回款": "CollectedPrepayment"
-                    , "回收回款": "CollectedRecoveries"}
-    return [[sourceMapping[x], acc] for (x, acc) in xs]
 
 
-def mkLiqProviderType(x):
-    match x:
-        case {"总额度": amt}:
-            return mkTag(("FixSupport"))
-        case {"日期":dp, "限额":amt}:
-            return mkTag(("ReplenishSupport", [mkDatePattern(dp),amt]))
-        case {}:
-            return mkTag(("UnLimit"))
-
-def mkLiqProvider(n, x):
-    match x:
-        case {"类型":"无限制","起始日":_sd, **p}: 
-            return {"liqName": n, "liqType": mkLiqProviderType({})
-                   , "liqBalance": None
-                   , "liqCredit": p.get("已提供", 0)
-                   , "liqStart": _sd}
-        case {"类型": _sp, "额度": _ab, "起始日":_sd, **p}: 
-            return {"liqName": n, "liqType": mkLiqProviderType(_sp)
-                   , "liqBalance": _ab
-                   , "liqCredit": p.get("已提供", 0)
-                   , "liqStart": _sd}
-        case _:
-            raise RuntimeError(f"无法匹配流动性支持类型：{n,x}")
 
 
-def mkComponent(x):
-    match x:
-        case {"贴现日": pricingDay, "贴现曲线": xs}:
-            return [pricingDay, {"tag": "PricingCurve", "contents": xs}]
-        case _:
-            None
-
-
-def mkLiq(x):
-    match x:
-        case {"正常余额折价": cf, "违约余额折价": df}:
-            return mkTag(("BalanceFactor", [cf, df]))
-        case {"贴现计价": df, "违约余额回收率": r}:
-            return mkTag(("PV", [df, r]))
-
-
-def mkAssumption(x):
-    match x:
-        case {"CPR": cpr} if isinstance(cpr,list):
-            return mkTag(("PrepaymentCPRCurve", cpr))
-        case {"CPR": cpr} :
-            return mkTag(("PrepaymentCPR", cpr))
-        case {"CPR调整": [*cprAdj,eDate]} :
-            return mkTag(("PrepaymentFactors" , mkTs("FactorCurveClosed",[cprAdj,eDate])))
-        case {"CDR": cdr}:
-            return mkTag(("DefaultCDR", cdr))
-        case {"CDR调整": [*cdrAdj,eDate]} :
-            return mkTag(("DefaultFactors" , mkTs("FactorCurveClosed",[cdrAdj,eDate])))
-        case {"回收": (rr, rlag)}:
-            return mkTag(("Recovery", (rr, rlag)))
-        case {"利率": [idx, rate]} if isinstance(rate, float):
-            return mkTag(("InterestRateConstant", [idx, rate]))
-        case {"利率": [idx, *rateCurve]}:
-            return mkTag(("InterestRateCurve", [idx, *rateCurve]))
-        case {"清仓": opts}:
-            return mkTag(("CallWhen",[mkCallOptions(co) for co in opts]))
-        case {"停止": d}:
-            return mkTag(("StopRunBy",d))
-
-def mkAccTxn(xs):
-    "AccTxn T.Day Balance Amount Comment"
-    if xs is None:
-        return None
-    else:
-        return [ mkTag(("AccTxn",x)) for x in xs]
-
-
-def mkComponent(x):
-    match x:
-        case {"贴现日": pricingDay, "贴现曲线": xs}:
-            return [pricingDay, {"tag": "PricingCurve", "contents": xs}]
-        case _:
-            None
-
-
-def mkLiq(x):
-    match x:
-        case {"正常余额折价": cf, "违约余额折价": df}:
-            return mkTag(("BalanceFactor", [cf, df]))
-        case {"贴现计价": df, "违约余额回收率": r}:
-            return mkTag(("PV", [df, r]))
-
-
-def mkCustom(x):
-    match x:
-        case {"常量":n}:
-            return mkTag(("CustomConstant",n))
-        case {"余额曲线":ts}:
-            return mkTag(("CustomCurve",mkTs("BalanceCurve",ts)))
-        case {"公式":ds}:
-            return mkTag(("CustomDS",mkDs(ds)))
-
-def mk(x):
-    match x:
-        case ["资产", assets]:
-            return {"assets": [mkAsset(a) for a in assets]}
-        case ["账户", accName, attrs]:
-            return {accName: mkAcc(accName, attrs)}
-        case ["费用", feeName, {"类型": feeType ,**fi}]:
-            return {feeName: {"feeName": feeName, "feeType": mkFeeType(feeType), "feeStart":fi.get("起算日",None)
-                             ,"feeDueDate":fi.get("计算日",None) , "feeDue": 0,
-                              "feeArrears": 0, "feeLastPaidDay": None}}
-        case ["债券", bndName, bnd]:
-            return mkBnd(bndName, bnd)
-        case ["归集规则", collection]:
-            return mkCollection(collection)
-        case ["清仓回购", calls]:
-            return mkCall(calls)
 
 
 @dataclass
@@ -199,13 +76,13 @@ class 信贷ABS:
 
     @property
     def json(self):
-        #cutoff, closing, first_pay = mkDate(self.日期)
         stated = False # self.日期.get("法定到期日",None) if len(self.日期)==4  # if isinstance(self.日期,dict) else self.日期[3]
         dists,collects,cleans = [ self.分配规则.get(wn,[]) for wn in ['未违约','回款后','清仓回购'] ]
         distsAs,collectsAs,cleansAs = [ [ mkWaterfall2(_action) for _action in _actions] for _actions in [dists,collects,cleans] ]
         distsflt,collectsflt,cleanflt = [ itertools.chain.from_iterable(x) for x in [distsAs,collectsAs,cleansAs] ]
         parsedDates = mkDate(self.日期)
         status = mkStatus(self.状态)
+        defaultStartDate = self.日期.get("起息日",None) or self.日期['归集日'][0]
         """
         get the json formatted string
         """
@@ -214,14 +91,11 @@ class 信贷ABS:
             "name": self.名称,
             "status": status,
             "pool":{"assets": [mkAsset(x) for x in self.资产池.get('清单',[])]
-                , "asOfDate": self.日期['封包日']
+                , "asOfDate": self.日期.get('封包日',None) or self.日期['归集日'][0]
                 , "issuanceStat": readIssuance(self.资产池)
                 , "futureCf":mkCf(self.资产池.get('归集表', []))},
             "bonds": functools.reduce(lambda result, current: result | current
                                       , [mk(['债券', bn, bo]) for (bn, bo) in self.债券]),
-            #"waterfall": {f"DistributionDay {status['tag']}":list(distsflt)
-            #            , "EndOfPoolCollection": list(collectsflt)
-            #            , "CleanUp": list(cleanflt)},
             "waterfall": mkWaterfall({},self.分配规则.copy()),
             "fees": functools.reduce(lambda result, current: result | current
                                      , [mk(["费用", feeName, feeO]) for (feeName, feeO) in self.费用]) if self.费用 else {},
@@ -232,7 +106,7 @@ class 信贷ABS:
         
         for fn, fo in _r['fees'].items():
             if fo['feeStart'] is None :
-                fo['feeStart'] = self.日期["起息日"]
+                fo['feeStart'] = defaultStartDate
 
         if hasattr(self, "自定义") and self.自定义 is not None:
             _r["custom"] = {}
@@ -249,7 +123,7 @@ class 信贷ABS:
         if hasattr(self, "流动性支持") and self.流动性支持 is not None:
             _providers = {}
             for (_k, _p) in self.流动性支持.items():
-                _providers[_k] = mkLiqProvider(_k, ( _p | {"起始日": self.日期["起息日"]}))
+                _providers[_k] = mkLiqProvider(_k, ( _p | {"起始日": defaultStartDate}))
             _r["liqProvider"] = _providers
 
         _dealType = identify_deal_type(_r)
@@ -293,10 +167,6 @@ class 信贷ABS:
         output['fees'] = {f: v.groupby('日期').agg({"余额": "min", "支付": "sum", "剩余支付": "min"})
                           for f, v in output['fees'].items()}
 
-        # aggregate liquidation provider 
-        #if 'liqProvider' in output:
-        #    output['liqProvider'] = {l:v.   
-        #                             for l,v in output['liqProvider'].items() }
 
         # aggregate accounts
         agg_acc = {}
