@@ -16,7 +16,12 @@ datePattern = {"月末":"MonthEnd"
 freqMap = {"每月": "Monthly"
     , "每季度": "Quarterly"
     , "每半年": "SemiAnnually"
-    , "每年": "Annually"}
+    , "每年": "Annually"
+    , "Monthly": "Monthly"
+    , "Quarterly": "Quarterly"
+    , "SemiAnnually": "SemiAnnually"
+    , "Annually": "Annually"
+    }
 
 baseMap = {"资产池余额": "CurrentPoolBalance"
            , "资产池期末余额": "CurrentPoolBalance"
@@ -32,8 +37,16 @@ baseMap = {"资产池余额": "CurrentPoolBalance"
            , "当期未付费用": "CurrentDueFee"
            }
 
-
-
+def mkLiq(x):
+    match x:
+        case {"正常余额折价": cf, "违约余额折价": df}:
+            return mkTag(("BalanceFactor", [cf, df]))
+        case {"CurrentFactor": cf, "DefaultFactor": df}:
+            return mkTag(("BalanceFactor", [cf, df]))
+        case {"贴现计价": df, "违约余额回收率": r}:
+            return mkTag(("PV", [df, r]))
+        case {"PV": df, "DefaultRecovery": r}:
+            return mkTag(("PV", [df, r]))
 
 def mkDatePattern(x):
     match x:
@@ -57,12 +70,17 @@ def mkDate(x):
     match x:
         case {"封包日":a, "起息日":b,"首次兑付日":c,"法定到期日":d,"收款频率":pf,"付款频率":bf} | \
              {"cutoff":a,"closing":b,"firstPay":c,"stated":d,"poolFreq":pf,"payFreq":bf}:
-            return mkTag(("PatternInterval"
-                          ,{"ClosingDate":[b,mkDatePattern(pf),d] ,"CutoffDate":[a,mkDatePattern(pf),d] 
-                           ,"FirstPayDate":[c,mkDatePattern(bf),d]}))
-        case {"回收期期初日":a, "起息日":b,"下次兑付日":c,"法定到期日":d,"收款频率":pf,"付款频率":bf} | \
-             {"cutoff":a,"closing":b,"nextPay":c,"stated":d,"poolFreq":pf,"payFreq":bf}:
-            return mkDate({"封包日":a, "起息日":b,"首次兑付日":c,"法定到期日":d,"收款频率":pf,"付款频率":bf})
+            firstCollection = x.get("首次归集日",b)
+            mr = x.get("循环结束日",None)
+            return mkTag(("PreClosingDates",[a,b,mr,d,[firstCollection,mkDatePattern(pf)],[c,mkDatePattern(bf)]]))
+        case {"归集日":(lastCollected,nextCollect),"兑付日":(pp,np),"法定到期日":c,"收款频率":pf,"付款频率":bf} | \
+             {"collect":(lastCollected,nextCollect),"pay":(pp,np),"stated":c,"poolFreq":pf,"payFreq":bf}:
+            mr = x.get("循环结束日",None)
+            return mkTag(("CurrentDates",[[lastCollected,pp],
+                                         mr,
+                                         c,
+                                         [nextCollect,mkDatePattern(pf)],
+                                         [np,mkDatePattern(bf)]]))
         case {"回款日":cdays, "分配日":ddays,"封包日":cutoffDate,"起息日":closingDate} | \
             {"poolCollection":cdays,"distirbution":ddays,"cutoff":cutoffDate,"closing":closingDate} :
             return mkTag(("CustomDates"
@@ -71,7 +89,7 @@ def mkDate(x):
                             ,closingDate
                             ,[ mkTag(("RunWaterfall",[dd,""])) for dd in ddays]]))
         case _:
-            raise RuntimeError(f"对于产品发行建模格式为：{'封包日':a, '起息日': b,'首次兑付日':c,'法定到期日':e,'收款频率':f,'付款频率':g} ")
+            raise RuntimeError(f"Failed to match:{x}")
 
 def mkFeeType(x):
     match x:
@@ -177,14 +195,6 @@ def isPre(x):
     except RuntimeError as e:
         return False 
 
-#def mkDealStatus(x):
-#    match x:
-#        case "":
-#            return ""
-#        case _ :
-#            raise RuntimeError(f"Failed to match on DealStatus: {x}")
-    
-
 
 def mkPre(p):
     dealStatusMap = {"摊还":"Current"
@@ -209,7 +219,7 @@ def mkPre(p):
             return mkTag(("IfAfterOnDate",_d))
         case ["<=",_d]:
             return mkTag(("IfBeforeOnDate",_d))
-        case ["状态",_st]:
+        case ["状态",_st] | ["status",_st]:
             return mkTag(("IfDealStatus",mkStatus(_st)))
         case ["同时满足",_p1,_p2] | ["all",_p1,_p2]:
             return mkTag(("And",mkPre(_p1),mkPre(_p2)))
@@ -279,19 +289,6 @@ def mkAcc(an,x):
             return mkAcc(an, x|{"计息":x.get("计息",None),"interest":x.get("interest",None)
                                 ,"记录":x.get("记录",None),"txn":x.get("txn",None)
                                 ,"类型":x.get("类型",None),"type":x.get("type",None)})
-        #case {"余额":b,"类型":t,"计息":i}|{"balance":b,"type":t,"interest":i}:
-        #    return mkAcc(an, x|{"记录":None,"txn":None})
-        #
-        #case {"余额":b,"类型":t,"记录":tx}|{"balance":b,"type":t,"txn":tx}:
-        #    return mkAcc(an, x|{"计息":None,"interest":None})
-        #
-        #case {"余额":b,"类型":t}|{"balance":b,"type":t}:
-        #    return mkAcc(an, x|{"计息":None,"interest":None, "记录":None,"txn":None})
-
-        #case {"余额":b}|{"balance":b}:
-        #    return mkAcc(an, x|{"计息":None,"interest":None
-        #                        ,"记录":None,"txn":None
-        #                        ,"类型":None,"type":None})
         case _ :
             raise RuntimeError(f"Failed to match account: {an},{x}")
  
@@ -309,36 +306,36 @@ def mkBondType(x):
         case _:
             raise RuntimeError(f"Failed to match bond type: {x}")
 
+
 def mkRateReset(x):
     match x:
-        case {"重置期间": interval, "起始": sdate}:
+        case {"重置期间": interval, "起始": sdate} | {"resetInterval": interval, "starts": sdate}:
             return mkTag(("ByInterval", [freqMap[interval], sdate]))
-        case {"重置期间": interval}:
+        case {"重置期间": interval} | {"resetInterval": interval }:
             return mkTag(("ByInterval", [freqMap[interval], None]))
-        case {"重置月份": monthOfYear}:
+        case {"重置月份": monthOfYear}| {"resetMonth":monthOfYear }:
             return mkTag(("MonthOfYear", monthOfYear))
         case _:
             raise RuntimeError(f"Failed to match:{x}: mkRateReset")
 
       
-
 def mkBondRate(x):
     indexMapping = {"LPR5Y": "LPR5Y", "LIBOR1M": "LIBOR1M"}
     match x:
         case {"浮动": [_index, Spread, resetInterval],"日历":dc} | \
-            {"Floater": [_index, Spread, resetInterval],"dayCount":dc}:
+            {"floater": [_index, Spread, resetInterval],"dayCount":dc}:
             return mkTag(("Floater", [indexMapping[_index]
                                     , Spread
                                     , mkRateReset(resetInterval)
                                     , dc
                                     , None
                                     , None]))
-        case {"浮动": [_index, Spread, resetInterval]} | {"Floater": [_index, Spread, resetInterval]}:
+        case {"浮动": [_index, Spread, resetInterval]} | {"floater": [_index, Spread, resetInterval]}:
             return mkBondRate(x | {"日历":DC.DC_ACT_365F.value
                                   ,"dayCount":DC.DC_ACT_365F.value})
         case {"固定": _rate, "日历":dc} | {"fix": _rate, "dayCount":dc}:
             return mkTag(("Fix",[_rate,dc]))
-        case {"固定": _rate} | {"fix": _rate}:
+        case {"固定": _rate} | {"Fixed": _rate}:
             return mkTag(("Fix",[_rate,DC.DC_ACT_365F.value]))
         case {"期间收益": _yield}:
             return mkTag(("InterestByYield",_yield))
@@ -379,11 +376,11 @@ def mkBnd(bn,x):
 
 def mkLiqMethod(x):
     match x:
-        case ["正常|违约",a,b]:
+        case ["正常|违约",a,b] | ["Cuurent|Defaulted",a,b]:
             return mkTag(("BalanceFactor",[a,b]))
-        case ["正常|拖欠|违约",a,b,c]:
+        case ["正常|拖欠|违约",a,b,c] | ["Cuurent|Delinquent|Defaulted",a,b,c]:
             return mkTag(("BalanceFactor2",[a,b,c]))
-        case ["贴现|违约",a,b]:
+        case ["贴现|违约",a,b] | ["PV|Defaulted",a,b]:
             return mkTag(("PV",[a,b]))
         case _ :
             raise RuntimeError(f"Failed to match {x}:mkLiqMethod")
@@ -391,9 +388,9 @@ def mkLiqMethod(x):
 
 def mkFeeCapType(x):
     match x:
-        case {"应计费用百分比": pct}:
+        case {"应计费用百分比": pct} | {"duePct": pct}:
             return mkTag(("DuePct",pct))
-        case {"应计费用上限": amt}:
+        case {"应计费用上限": amt} | {"dueCapAmt": amt}:
             return mkTag(("DueCapAmt",amt))
         case _ :
             raise RuntimeError(f"Failed to match {x}:mkFeeCapType")
@@ -401,7 +398,7 @@ def mkFeeCapType(x):
 
 def mkPDA(x):
     match x:
-        case {"公式": ds}:
+        case {"公式": ds} | {"formula": ds}:
             return mkTag(("DS",mkDs(ds)))
         case _ :
             raise RuntimeError(f"Failed to match {x}:mkPDA")
@@ -409,9 +406,9 @@ def mkPDA(x):
 
 def mkAccountCapType(x):
     match x:
-        case {"余额百分比": pct}:
+        case {"余额百分比": pct} | {"balPct": pct}:
             return mkTag(("DuePct",pct))
-        case {"金额上限": amt}:
+        case {"金额上限": amt} | {"balCapAmt": amt}:
             return mkTag(("DueCapAmt",amt))
         case _ :
             raise RuntimeError(f"Failed to match {x}:mkAccountCapType")
@@ -419,19 +416,18 @@ def mkAccountCapType(x):
 
 def mkTransferLimit(x):
     match x:
-        case {"余额百分比": pct}:
+        case {"余额百分比": pct} | {"balPct": pct}:
             return mkTag(("DuePct",pct))
-        case {"金额上限": amt}:
+        case {"金额上限": amt} | {"balCapAmt": amt}:
             return mkTag(("DueCapAmt",amt))
         case {"公式": "ABCD" }:
             return mkTag(("Formula","ABCD"))
-        case {"公式": formula}:
+        case {"公式": formula} | {"formula": formula}:
             return mkTag(("DS",mkDs(formula)))
         case _:
             raise RuntimeError(f"Failed to match :{x}:mkTransferLimit")
 
 
-
 def mkAction(x):
     match x:
         case ["账户转移", source, target] | ["transfer", source, target]:
@@ -498,28 +494,30 @@ def mkWaterfall2(x):
 
 def mkStatus(x):
     match x : 
-        case "摊销":
+        case "摊销"|"Amortizing":
             return mkTag(("Amortizing"))
-        case "循环":
+        case "循环"|"Revolving":
             return mkTag(("Revolving"))
-        case "加速清偿":
+        case "加速清偿"|"Accelerated":
             return mkTag(("DealAccelerated",None))
-        case "违约":
+        case "违约"|"Defaulted":
             return mkTag(("DealDefaulted",None))
-        case "结束":
+        case "结束"|"Ended":
             return mkTag(("Ended"))
+        case "设计"|"PreClosing":
+            return mkTag(("PreClosing"))
         case _:
             raise RuntimeError(f"Failed to match :{x}:mkStatus")
 
 def mkWhenTrigger(x):
     match x:
-        case "回收后":
+        case "回收后"|"BeforeCollect":
             return "BeginCollectionWF"
-        case "回收动作后":
+        case "回收动作后"|"AfterCollect":
             return "EndCollectionWF"
-        case "分配前":
+        case "分配前"|"BeforeDistribution":
             return "BeginDistributionWF"
-        case "分配后":
+        case "分配后"|"AfterDistribution":
             return "EndDistributionWF"
         case _:
             raise RuntimeError(f"Failed to match :{x}:mkWhenTrigger")
@@ -560,17 +558,17 @@ def mkTrigger(x):
             return mkTag(("AfterDate",_d))
         case [">=",_d]:
             return mkTag(("AfterOnDate",_d))
-        case ["到期日未兑付",_bn]:
+        case ["到期日未兑付",_bn] | ["passMaturity",_bn]:
             return mkTag(("PassMaturityDate",_bn))
         #case ("目标摊还不足",bn):
         #   return mkTag(("PrinShortfall",bn))
         #case ["到期日未兑付",bn]:
         #    return mkTag(("MissMatureDate",bn))
-        case ["所有满足",*trgs]:
+        case ["所有满足",*trgs] | ["all",*trgs]:
             return mkTag(("AllTrigger",[ mkTrigger(t) for t in trgs ]))
-        case ["任一满足",*trgs]:
+        case ["任一满足",*trgs] | ["any",*trgs]:
             return mkTag(("AnyTrigger",[ mkTrigger(t) for t in trgs ]))
-        case ["一直",b]:
+        case ["一直",b] | ["always",b]:
             return mkTag(("Always",b))
         case _:
             raise RuntimeError(f"Failed to match :{x}:mkTrigger")
@@ -578,25 +576,17 @@ def mkTrigger(x):
 
 def mkTriggerEffect(x):
     match x:
-        case ("新状态",s):
+        case ("新状态",s) | ("newStatus",s):
             return mkTag(("DealStatusTo", mkStatus(s)))
-        case ["计提费用",*fn]:
+        case ["计提费用",*fn] | ["accureFees",*fn]:
             return mkTag(("DoAccrueFee", fn))
-        case ["新增事件",trg]:
+        case ["新增事件",trg] | ["newTrigger",trg]:
             return mkTag(("AddTrigger", mkTrigger(trg)))
-        case ["结果",*efs]:
+        case ["结果",*efs] | ["Effects",*efs]:
             return mkTag(("TriggerEffects",[mkTriggerEffect(e) for e in efs]))
         case _:
             raise RuntimeError(f"Failed to match :{x}:mkTriggerEffect")
 
-#def mkActionWhen(x):
-#    match x:
-#        case "未违约"|"兑付日":
-#        
-#        case _:
-#            raise RuntimeError(f"Failed to match ActionWhen:{x}")
-
-
 
 def mkWaterfall(r, x):
     mapping = {
@@ -605,24 +595,27 @@ def mkWaterfall(r, x):
         "循环":"Revolving",
         "加速清偿":"DealAccelerated",
         "违约":"DealDefaulted",
+        "未设立":"PreClosing",
     }
     if len(x)==0:
         return {k:list(v)  for k,v in r.items()}
     _k,_v = x.popitem()
     _w_tag = None
     match _k:
-        case ("兑付日","加速清偿"):
+        case ("兑付日","加速清偿") | ("amortizing","accelerated"):
             _w_tag = f"DistributionDay (DealAccelerated Nothing)"
-        case ("兑付日","违约"):
+        case ("兑付日","违约") | ("amortizing","defaulted"):
             _w_tag = f"DistributionDay (DealDefaulted Nothing)"
-        case ("兑付日",_st):
-            _w_tag = f"DistributionDay {mapping[_st]}"
-        case "兑付日" | "未违约":
+        case ("兑付日",_st) | ("amortizing",_st):
+            _w_tag = f"DistributionDay {mapping.get(_st,_st)}"
+        case "兑付日" | "未违约" | "amortizing":
             _w_tag = f"DistributionDay Amortizing"
-        case "清仓回购":
+        case "清仓回购" | "cleanUp":
             _w_tag = "CleanUp"
-        case "回款日" | "回款后" :
+        case "回款日" | "回款后" | "endOfCollection":
             _w_tag = f"EndOfPoolCollection"
+        case "设立日" | "closingDay":
+            _w_tag = f"OnClosingDay"
         case _:
             raise RuntimeError(f"Failed to match :{x}:mkWaterfall")
     r[_w_tag] = itertools.chain.from_iterable([mkWaterfall2(_a) for _a in _v])
@@ -631,10 +624,12 @@ def mkWaterfall(r, x):
 
 def mkAssetRate(x):
     match x:
-        case ["固定",r]:
+        case ["固定",r] | ["Fixed",r]:
             return mkTag(("Fix",r))
         case ["浮动",r,{"基准":idx,"利差":spd,"重置频率":p}]:
             return mkTag(("Floater",[idx,spd,r,freqMap[p],None]))
+        case ["floater",r,{"index":idx,"spread":spd,"reset":p}]:
+            return mkTag(("Floater",[idx,spd,r,freqMap[p],None]))
         case _ :
             raise RuntimeError(f"Failed to match {x}:mkAssetRate")
 
@@ -653,15 +648,24 @@ def mkAmortPlan(x)->dict:
 
 
 def mkAsset(x):
-    _statusMapping = {"正常": mkTag(("Current")), "违约": mkTag(("Defaulted",None))}
+    _statusMapping = {"正常": mkTag(("Current"))
+                    , "违约": mkTag(("Defaulted",None))
+                    , "Current":mkTag(("Current"))
+                    , "Defaulted": mkTag(("Defaulted",None))
+                    }
     match x:
         case ["按揭贷款"
-            ,{"放款金额": originBalance, "放款利率": originRate, "初始期限": originTerm
-                  ,"频率": freq, "类型": _type, "放款日": startDate}
-            ,{"当前余额": currentBalance
-             ,"当前利率": currentRate
-             ,"剩余期限": remainTerms
-             ,"状态": status}]:
+              ,{"放款金额": originBalance, "放款利率": originRate, "初始期限": originTerm ,"频率": freq, "类型": _type, "放款日": startDate}
+              ,{"当前余额": currentBalance
+                ,"当前利率": currentRate
+                ,"剩余期限": remainTerms
+                ,"状态": status}] | \
+            ["Mortgage"
+            ,{"originBalance": originBalance, "originRate": originRate, "originTerm": originTerm ,"freq": freq, "type": _type, "originDate": startDate}
+            ,{"currentBalance": currentBalance
+             ,"currentRate": currentRate
+             ,"remainTerm": remainTerms
+             ,"status": status}]:
             return mkTag(("Mortgage",[
                                       {"originBalance": originBalance,
                                       "originRate": mkAssetRate(originRate),
@@ -669,7 +673,7 @@ def mkAsset(x):
                                       "period": freqMap[freq],
                                       "startDate": startDate,
                                       "prinType": mkAmortPlan(_type)
-                                      } |mkTag("MortgageOriginalInfo"),
+                                      } | mkTag("MortgageOriginalInfo"),
                                      currentBalance,
                                      currentRate,
                                      remainTerms,
@@ -680,7 +684,14 @@ def mkAsset(x):
             ,{"当前余额": currentBalance
              ,"当前利率": currentRate
              ,"剩余期限": remainTerms
-             ,"状态": status}]:
+             ,"状态": status}] \
+            |["loan"
+            ,{"originBalance": originBalance, "originRate": originRate, "originTerm": originTerm
+                  ,"freq": freq, "type": _type, "originDate": startDate}
+            ,{"currentBalance": currentBalance
+             ,"currentRate": currentRate
+             ,"remainTerm": remainTerms
+             ,"status": status}]:
             return mkTag(("PersonalLoan",[
                                       {"originBalance": originBalance,
                                       "originRate": mkAssetRate(originRate),
@@ -696,9 +707,11 @@ def mkAsset(x):
         case ["分期"
             ,{"放款金额": originBalance, "放款费率": originRate, "初始期限": originTerm
                   ,"频率": freq, "类型": _type, "放款日": startDate}
-            ,{"当前余额": currentBalance
-             ,"剩余期限": remainTerms
-             ,"状态": status}]:
+            ,{"状态": status}] \
+            |["installment"
+            ,{"originBalance": originBalance, "feeRate": originRate, "originTerm": originTerm
+                  ,"freq": freq, "type": _type, "originDate": startDate}
+            ,{"status": status}]:
             return mkTag(("Installment",[
                                       {"originBalance": originBalance,
                                       "originRate": mkAssetRate(originRate),
@@ -707,10 +720,8 @@ def mkAsset(x):
                                       "startDate": startDate,
                                       "prinType": mkAmortPlan(_type)
                                       } | mkTag("LoanOriginalInfo"),
-                                     currentBalance,
-                                     remainTerms,
+                                     0.0,
                                      _statusMapping[status]]))       
- 
 
         case _ :
             raise RuntimeError(f"Failed to match {x}:mkAsset")
@@ -726,22 +737,25 @@ def identify_deal_type(x):
             return "IDeal"
         case _ :
             return "MDeal"
+        #case _ :
+        #    raise RuntimeError(f"Failed to identify deal type {x}")
+
 
 def mkCallOptions(x):
     match x:
-        case {"资产池余额": bal}:
+        case {"资产池余额": bal} | {"poolBalance": bal}:
             return mkTag(("PoolBalance", bal))
-        case {"债券余额": bal}:
+        case {"债券余额": bal} | {"bondBalance": bal}:
             return mkTag(("BondBalance", bal))
-        case {"资产池余额剩余比率": factor}:
+        case {"资产池余额剩余比率": factor} | {"poolFactor": factor}:
             return mkTag(("PoolFactor", factor))
-        case {"债券余额剩余比率": factor}:
+        case {"债券余额剩余比率": factor} | {"bondFactor": factor}:
             return mkTag(("BondFactor", factor))
-        case {"指定日之后": d}:
+        case {"指定日之后": d} | {"afterDate": d}:
             return mkTag(("AfterDate", d))
-        case {"任意满足": xs}:
+        case {"任意满足": xs} | {"or": xs}:
             return mkTag(("Or", xs))
-        case {"全部满足": xs}:
+        case {"全部满足": xs} | {"and": xs}:
             return mkTag(("And", xs))
         case _ :
             raise RuntimeError(f"Failed to match {x}:mkCallOptions")
@@ -755,31 +769,136 @@ def mkAssumption(x) -> dict:
             return mkTag(("DefaultVec", cdr))
         case {"CPR": cpr} :
             return mkTag(("PrepaymentCPR", cpr))
-        case {"CPR调整": [*cprAdj,eDate]} :
-            return mkTag(("PrepaymentFactors" , mkTs("FactorCurveClosed",[cprAdj,eDate])))
+        case {"CPR调整": [*cprAdj,ed]} | {"CPRAdjust":[*cprAdj,ed]}:
+            return mkTag(("PrepaymentFactors" , mkTs("FactorCurveClosed",[cprAdj,ed])))
         case {"CDR": cdr}:
             return mkTag(("DefaultCDR", cdr))
-        case {"CDR调整": [*cdrAdj,eDate]} :
-            return mkTag(("DefaultFactors" , mkTs("FactorCurveClosed",[cdrAdj,eDate])))
-        case {"回收": (rr, rlag)}:
+        case {"CDR调整": [*cdrAdj,ed]} | {"CDRAdjust":[*cdrAdj,ed]}:
+            return mkTag(("DefaultFactors" , mkTs("FactorCurveClosed",[cdrAdj,ed])))
+        case {"回收": (rr, rlag)} | {"Recovery":(rr,rlag)}:
             return mkTag(("Recovery", (rr, rlag)))
         case {"利率": [idx, rate]} if isinstance(rate, float):
             return mkTag(("InterestRateConstant", [idx, rate]))
-        case {"利率": [idx, *rateCurve]}:
+        case {"Rate": [idx, rate]} if isinstance(rate, float):
+            return mkTag(("InterestRateConstant", [idx, rate]))
+        case {"利率": [idx, *rateCurve]} | {"Rate":[idx, *rateCurve]}:
             return mkTag(("InterestRateCurve", [idx, *rateCurve]))
-        case {"清仓": opts}:
+        case {"清仓": opts} | {"CleanUp":opts}:
             return mkTag(("CallWhen",[mkCallOptions(co) for co in opts]))
-        case {"停止": d}:
+        case {"停止": d} | {"StopAt":d}:
             return mkTag(("StopRunBy",d))
         case _ :
             raise RuntimeError(f"Failed to match {x}:Assumption")
 
+def mkAssumpList(xs):
+    return [mkAssumption(x) for x in xs]
+
+def mkAssumption2(x) -> dict:
+    match x:
+        case (assetAssumpList,dealAssump) if isinstance(x,tuple):
+            return mkTag(("ByIndex",[[(ids,mkAssumpList(aps)) for ids,aps in assetAssumpList], mkAssumpList(dealAssump)]))
+        case xs if isinstance(xs, list):
+            return mkTag(("PoolLevel", mkAssumpList(xs)))
+        case None:
+            return None
+        case _:
+            raise RuntimeError(f"Failed to match {x}:mkAssumption2")
+
 def mkPool(x):
     mapping = {"LDeal":"LPool","MDeal":"MPool","IDeal":"IPool"}
     match x:
-        case {"清单":assets,"封包日":coday}:
-            _pool = {"assets": [mkAsset(a) for a in assets],"asOfDate": coday}
+        case {"清单":assets,"封包日":d} | {"assets":assets,"cutoffDate":d}:
+            _pool = {"assets": [mkAsset(a) for a in assets],"asOfDate": d}
             _pool_asset_type = identify_deal_type({"pool":_pool})
             return mkTag((mapping[_pool_asset_type], _pool))
         case _:
             raise RuntimeError(f"Failed to match {x}:mkPool")
+
+def mkCustom(x):
+    match x:
+        case {"常量":n} | {"Constant":n}:
+            return mkTag(("CustomConstant",n))
+        case {"余额曲线":ts} | {"BalanceCurve":ts}:
+            return mkTag(("CustomCurve",mkTs("BalanceCurve",ts)))
+        case {"公式":ds} | {"Formula":ds}:
+            return mkTag(("CustomDS",mkDs(ds)))
+
+def mkLiqProvider(n, x):
+    match x:
+        case {"类型":"无限制","起始日":_sd, **p} \
+            | {"type":"Unlimited","start":_sd, **p}: 
+            return {"liqName": n, "liqType": mkLiqProviderType({})
+                   , "liqBalance": None
+                   , "liqCredit": p.get("已提供", 0)
+                   , "liqStart": _sd}
+        case {"类型": _sp, "额度": _ab, "起始日":_sd, **p} \
+            | {"type": _sp, "lineOfCredit": _ab, "start":_sd, **p}: 
+            return {"liqName": n, "liqType": mkLiqProviderType(_sp)
+                   , "liqBalance": _ab
+                   , "liqCredit": p.get("已提供", 0)
+                   , "liqStart": _sd}
+        case _:
+            raise RuntimeError(f"无法匹配流动性支持类型：{n,x}")
+
+def mkCf(x):
+    if len(x)==0:
+        return None
+    else:
+        return [ mkTag(("MortgageFlow",_x+([0.0]*5))) for _x in x]
+
+def mkCollection(xs):
+    sourceMapping = {"利息回款": "CollectedInterest"
+                    , "本金回款": "CollectedPrincipal"
+                    , "早偿回款": "CollectedPrepayment"
+                    , "回收回款": "CollectedRecoveries"
+                    ,"CollectedInterest": "CollectedInterest"
+                    , "CollectedPrincipal": "CollectedPrincipal"
+                    , "CollectedPrepayment": "CollectedPrepayment"
+                    , "CollectedRecoveries": "CollectedRecoveries"
+                    }
+    return [[sourceMapping[x], acc] for (x, acc) in xs]
+
+def mkAccTxn(xs):
+    "AccTxn T.Day Balance Amount Comment"
+    if xs is None:
+        return None
+    else:
+        return [ mkTag(("AccTxn",x)) for x in xs]
+
+
+def mk(x):
+    match x:
+        case ["资产", assets]:
+            return {"assets": [mkAsset(a) for a in assets]}
+        case ["账户", accName, attrs] | ["account", accName, attrs]:
+            return {accName: mkAcc(accName, attrs)}
+        case ["费用", feeName, {"类型": feeType ,**fi}] \
+            | ["fee", feeName, {"type": feeType ,**fi}] :
+            return {feeName: {"feeName": feeName, "feeType": mkFeeType(feeType), "feeStart":fi.get("起算日",None)
+                             ,"feeDueDate":fi.get("计算日",None) , "feeDue": 0,
+                              "feeArrears": 0, "feeLastPaidDay": None}}
+        case ["债券", bndName, bnd] | ["bond", bndName, bnd]:
+            return mkBnd(bndName, bnd)
+        case ["归集规则", collection]:
+            return mkCollection(collection)
+        #case ["清仓回购", calls]:
+        #    return mkCall(calls)
+
+
+def mkComponent(x):
+    match x:
+        case {"贴现日": pricingDay, "贴现曲线": xs}:
+            return [pricingDay, {"tag": "PricingCurve", "contents": xs}]
+        case {"PVDate": pricingDay, "PVCurve": xs}:
+            return [pricingDay, {"tag": "PricingCurve", "contents": xs}]
+        case _:
+            None
+
+def mkLiqProviderType(x):
+    match x:
+        case {"总额度": amt} | {"Total": amt}:
+            return mkTag(("FixSupport"))
+        case {"日期":dp, "限额":amt} | {"Reset":dp, "Quota":amt}:
+            return mkTag(("ReplenishSupport", [mkDatePattern(dp),amt]))
+        case {}:
+            return mkTag(("UnLimit"))            
\ No newline at end of file
